var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import * as util from 'redux-service-util';
import * as todos from '../';
import * as service from '../todos-service';
function createTodo(id) {
    if (id === void 0) { id = 'abc123'; }
    return {
        id: id,
        title: "a testing todo: " + id,
        completed: false,
    };
}
jest.mock('redux-service-util', function () { return ({
    actionAsync: jest.fn(),
    reducerAsyncActions: jest.fn(),
    reducerPending: jest.fn(),
    reducerFail: jest.fn(),
    typePending: jest.fn().mockImplementation(function (value) { return value + ".pending"; }),
    typeFail: jest.fn().mockImplementation(function (value) { return value + ".fail"; }),
    typeOK: jest.fn().mockImplementation(function (value) { return value + ".ok"; }),
    createReducer: jest.fn(),
    apiStateOK: jest.fn(),
    withoutApi: jest.fn(),
    asValues: jest.fn(),
    apiOnly: jest.fn(),
}); });
var actionAsyncMock = util.actionAsync;
var asValuesMock = util.asValues;
var withoutApiMock = util.withoutApi;
var apiOnlyMock = util.apiOnly;
var apiStateOK = util.apiStateOK;
describe('todo actions', function () {
    var _a, _b;
    var todo = createTodo('abc123');
    var params = (_a = {},
        _a[todos.types.todoAdd] = todo,
        _a[todos.types.todoDelete] = 'abc123',
        _a[todos.types.todoComplete] = todo,
        _a[todos.types.todoIncomplete] = todo,
        _a);
    var asyncOpts = (_b = {},
        _b[todos.types.todoAdd] = { updating: 'new' },
        _b[todos.types.todosList] = { fetched: true },
        _b[todos.types.todoDelete] = { updating: 'abc123' },
        _b[todos.types.todoComplete] = { updating: 'abc123' },
        _b[todos.types.todoIncomplete] = { updating: 'abc123' },
        _b);
    // if actions are very templated invocations of ActionAsync, tests can be done using an iteration over the action types
    Object.entries(todos.types).forEach(function (_a) {
        var key = _a[0];
        test(key + ": invokes actionAsync with the correct parameters", function () {
            var param = params[key];
            var opt = asyncOpts[key];
            todos.actions[key](param);
            expect(actionAsyncMock.mock.calls).toHaveLength(1);
            var serviceHandlers = service;
            var opts = __assign({ type: key, handler: serviceHandlers[key] }, opt);
            expect(actionAsyncMock.mock.calls[0][0]).toEqual(opts);
            expect(actionAsyncMock.mock.calls[0][1]).toBe(param);
        });
    });
});
describe('reducer map special case handling', function () {
    var state = __assign({ abc123: createTodo('abc123'), def456: createTodo('def456'), hij678: createTodo('hij678') }, util.INITIAL_ASYNC_STATE);
    var action = { type: 'action', payload: { id: 'def456' } };
    test('reducerMap: todoDelete.ok deletes the target item from the state', function () {
        apiStateOK.mockReturnValue({ key: 'value' });
        var result = todos.todoDeleteOK(state, action);
        var value = state.def456, expected = __rest(state, ["def456"]);
        var expectedWithApi = __assign({}, expected, { api: { key: 'value' } });
        expect(result).toEqual(expectedWithApi);
    });
    test('reducerMap: todoDelete.ok handles a non-existent key', function () {
        apiStateOK.mockReturnValue({ key: 'value' });
        var result = todos.todoDeleteOK(state, { type: 'action', payload: { id: 'def454' } });
        var expectedWithApi = __assign({}, state, { api: { key: 'value' } });
        expect(result).toEqual(expectedWithApi);
    });
});
describe('selectors', function () {
    test('allTodos: invokes withoutApi util function', function () {
        var state = { todos: __assign({ a: '1' }, util.INITIAL_ASYNC_STATE) };
        todos.select.allTodos(state);
        expect(withoutApiMock.mock.calls).toHaveLength(1);
        expect(withoutApiMock.mock.calls[0][0]).toBe(state.todos);
    });
    test('allTodosAsArray: invokes withoutApi util function', function () {
        var state = { todos: __assign({ a: '1' }, util.INITIAL_ASYNC_STATE) };
        todos.select.allTodosAsArray(state);
        expect(asValuesMock.mock.calls).toHaveLength(1);
        expect(asValuesMock.mock.calls[0][0]).toBe(state.todos);
    });
    test('apiState: invokes apiOnly util function', function () {
        var state = { todos: __assign({ a: '1' }, util.INITIAL_ASYNC_STATE) };
        todos.select.apiState(state);
        expect(apiOnlyMock.mock.calls).toHaveLength(1);
        expect(apiOnlyMock.mock.calls[0][0]).toBe(state.todos);
    });
});
//# sourceMappingURL=todos-state.test.js.map